@@ -6,7 +6,7 @@
 ;; Author:     FSF (see vc.el for full credits)
 ;; Maintainer: Andre Spiegel <spiegel@gnu.org>
 
-;; $Id: vc-hooks.el,v 1.14 2007/09/28 19:04:24 scop Exp $
+;; $Id: vc-hooks.el,v 1.16 2009/03/02 16:33:11 scop Exp $
 
 ;; This file is part of GNU Emacs.
 
@@ -443,18 +443,32 @@ For registered files, the possible values are:
           (vc-file-setprop file 'vc-checkout-model
                            (vc-call checkout-model file)))))
 
+
+;; XEmacs change: added for our with-parsed-tramp-file-name change below
+(eval-when-compile
+  (require 'tramp))
+
 (defun vc-user-login-name (file)
   "Return the name under which the user accesses the given FILE."
   (or (and (eq (string-match tramp-file-name-regexp file) 0)
            ;; tramp case: execute "whoami" via tramp
-           (let ((default-directory (file-name-directory file)))
-             (with-temp-buffer
-               (if (not (zerop (process-file "whoami" nil t)))
-                   ;; fall through if "whoami" didn't work
-                   nil
-                 ;; remove trailing newline
-                 (delete-region (1- (point-max)) (point-max))
-                 (buffer-string)))))
+           ;; XEmacs change: we don't have process-file or call-process for
+           ;; which we could apply the tramp handler, use tramp directly.
+           ;; TODO: this should work with tramp 2.0, but won't work with >= 2.1
+           (condition-case nil
+               (progn
+                 (require 'tramp)
+                 (with-parsed-tramp-file-name file nil
+                   (if (not (zerop (tramp-send-command-and-check
+                                    multi-method method user host "whoami")))
+                       nil
+                     ;; First line should be the output of "whoami"
+                     (goto-char (point-min))
+                     (read (current-buffer)))))
+             (error
+              (message (concat "Could not resolve login name using tramp, "
+                               "falling back to local (known bug: does not "
+                               "work with tramp >= 2.1)")))))
       ;; normal case
       (user-login-name)
       ;; if user-login-name is nil, return the UID as a string
@@ -921,7 +935,7 @@ Used in `find-file-not-found-functions'."
 (defconst vc-menu
   '("Version Control"
     :filter vc-menu-filter
-    [""                           vc-next-action          buffer-file-name nil]
+    ["Next Action on"             vc-next-action          buffer-file-name nil]
     ["Update to Latest Version"   vc-update               vc-mode]
     ["Revert to Base Version"     vc-revert-buffer        vc-mode]
     ["Undo Last Check-In"         vc-cancel-version       vc-mode]
@@ -942,48 +956,15 @@ Used in `find-file-not-found-functions'."
 
 (defun vc-menu-filter (menu-items)
   (let* ((result menu-items)		; modify in-place
-	 (case-fold-search t)
-	 (type (vc-backend buffer-file-name))
 	 ;; VC works with full path!
-	 (file (if buffer-file-name
-		   buffer-file-name
-		 (expand-file-name (buffer-name))))
-	 op owner item status)
-    (setq op (cond ((null type)
-		    "Register File")
-		   ((eq type 'CVS)
-		    (setq status
-			  (vc-file-getprop buffer-file-name 'cvs-status))
-		    (if status
-			(cdr (assoc status
-				    '(("Locally Modified" . "Commit")
-				      ("Needs Merge" . "Merge with repository")
-				      ("Up-to-date" . "Do nothing to")
-				      ("Needs Checkout" . "Update"))))
-		      ;; #### - we're not gonna call cvs status just to
-		      ;; post a lousy menu...that's insane!
-		      "Next action on" 
-		      ))
-		   ;; these are all for RCS and SCCS
-		   ((not (setq owner (vc-locking-user file)))
-		    ;; VC locking user is faster now. vc-file-owner
-		    ;; is superceded by incompatible FSF version (JV).
-		    "Check out File")
-		   ((not (string-equal owner (user-login-name)))
-		    "Steal File Lock")
-		   (t "Check in File")))
+	 (file (or buffer-file-name (expand-file-name (buffer-name))))
+	 item)
     ;; We do not want the full name in the menu
     (setq file (file-name-nondirectory file))
     (while (setq item (pop menu-items))
       (and (vectorp item)
-	   (cond ((eq 'vc-next-action (aref item 1))
-		  (aset item 0 op)
-		  (aset item 3 file))
-		 ((eq 'vc-file-status (aref item 1))
-		  (aset item 2 (eq 'CVS type))
-		  (aset item 3 file))
-		 ((> (length item) 3)
-		  (aset item 3 file)))))
+	   (> (length item) 3)
+	   (aset item 3 file)))
     result))
 
 (and (featurep 'menubar)
